package com.loafle.overflow.module.email.service;

import com.loafle.overflow.module.email.dao.EmailAuthDAO;
import com.loafle.overflow.module.email.model.EmailAuth;
import com.loafle.overflow.module.member.dao.MemberDAO;
import com.loafle.overflow.module.member.model.Member;
import com.loafle.overflow.module.meta.model.MetaMemberStatus;
import org.apache.commons.codec.binary.Base64;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.FileSystemResource;
import org.springframework.mail.MailException;
import org.springframework.mail.SimpleMailMessage;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.mail.javamail.MimeMessageHelper;
import org.springframework.stereotype.Service;

import javax.crypto.Cipher;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import javax.mail.MessagingException;
import javax.mail.internet.MimeMessage;
import java.io.File;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.Date;
import java.util.List;

/**
 * Created by geek on 17. 6. 28.
 */
@Service("EmailAuthService")
public class EmailAuthService {

    @Autowired
    private JavaMailSender mailSender;

    @Autowired
    private EmailAuthDAO emailAuthDAO;

    @Autowired
    private MemberDAO memberDAO;

    private String key = "loafle@RandomKey";
    private String initVector = "loafleInitVector";

    public EmailAuth sendEmailByMember(long memberId, String memberEmail) throws UnsupportedEncodingException {

        EmailAuth auth = new EmailAuth();
        auth.setMember(new Member(memberId));
        // Todo AuthKey Generation
        String en = this.encrypt(memberEmail);
        auth.setEmailAuthKey(en);

        String encode = URLEncoder.encode(en, "UTF-8");

//        System.out.println("encode = [" + encode + "]");
        try {
            this.sendSimpleEmail(memberEmail, "Confirm Email", "http://127.0.0.1:19080/account/check_email?key="+ encode +"\r\nConfirm Email");
        }catch (MailException e) {
            e.printStackTrace();
        }

        this.emailAuthDAO.save(auth);

        return auth;
    }

    public EmailAuth read(long id) {
        return this.emailAuthDAO.findOne(id);
    }

    public EmailAuth readByAuthKey(String authKey) throws UnsupportedEncodingException {
        System.out.println("authKey = [" + authKey + "]");
        String deStr = URLDecoder.decode(authKey, "UTF-8");
        System.out.println("deStr = [" + deStr + "]");

        EmailAuth auth = this.emailAuthDAO.findByEmailAuthKey(deStr);

        if (auth != null) {
            auth.setAuthConfirmDate(new Date());
            this.emailAuthDAO.save(auth);
            auth.getMember().setStatus(new MetaMemberStatus((short)2));
            this.memberDAO.save(auth.getMember());
        }
        return auth;
    }

    // dZQgXM1o/Cx48X8DM+6ec/oPfqA2l/LdWtijOZ2EnWk=
    // dZQgXM1o/Cx48X8DM 6ec/oPfqA2l/LdWtijOZ2EnWk=

    public List<EmailAuth> readByMember(long memberId) {
        return this.emailAuthDAO.findByMember(new Member(memberId));
    }

    public EmailAuth modify(EmailAuth emailAuth) {
        return this.emailAuthDAO.save(emailAuth);
    }

    private void sendSimpleEmail(String to, String sub, String message) throws MailException {

        SimpleMailMessage message1 = new SimpleMailMessage();
        message1.setTo(to);
        message1.setSubject(sub);
        message1.setText(message);
        message1.setFrom("geek@loafle.com");

        mailSender.send(message1);
    }

    private void sendMailWithAttachment(String to, String sub, String text, String path) throws MessagingException {
        MimeMessage message = mailSender.createMimeMessage();
        // pass 'true' to the constructor to create a multipart message
        MimeMessageHelper helper = new MimeMessageHelper(message, true);

        helper.setTo(to);
        helper.setSubject(sub);
        helper.setText(text);
        helper.setFrom("geek@loafle.com");
        FileSystemResource file = new FileSystemResource(new File(path));
        helper.addAttachment("Invoice", file);

        mailSender.send(message);
    }

    private String encrypt(String value) {
        try {
            IvParameterSpec iv = new IvParameterSpec(initVector.getBytes("UTF-8"));

            SecretKeySpec skeySpec = new SecretKeySpec(key.getBytes("UTF-8"), "AES");

            Cipher cipher = Cipher.getInstance("AES/CBC/PKCS5PADDING");
            cipher.init(Cipher.ENCRYPT_MODE, skeySpec, iv);

            byte[] encrypted = cipher.doFinal(value.getBytes());
            System.out.println("encrypted string: "
                    + Base64.encodeBase64String(encrypted));

            return Base64.encodeBase64String(encrypted);
        } catch (Exception ex) {
            ex.printStackTrace();
        }

        return null;
    }

    public String decrypt(String encrypted) {
        try {
            IvParameterSpec iv = new IvParameterSpec(initVector.getBytes("UTF-8"));
            SecretKeySpec skeySpec = new SecretKeySpec(key.getBytes("UTF-8"), "AES");

            Cipher cipher = Cipher.getInstance("AES/CBC/PKCS5PADDING");
            cipher.init(Cipher.DECRYPT_MODE, skeySpec, iv);

            byte[] original = cipher.doFinal(Base64.decodeBase64(encrypted));

            return new String(original);
        } catch (Exception ex) {
            ex.printStackTrace();
        }

        return null;
    }

}
